//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Execution
{
    using System;
    using System.Globalization;

    /// <summary>
    /// Defines methods and properties to get the formated exception messages.
    /// </summary>
    internal static class Messages
    {
        /// <summary>
        /// Gets the error message to show to user when stop requested.
        /// </summary>
        public static string TerminatedMessage
        {
            get { return "Terminated by user"; }
        }

        /// <summary>
        /// Gets the error message when the job has zero activity instances.
        /// </summary>
        public static string ZeroActivityInstanceError
        {
            get { return "Job does not contain ActivityInstance object"; }
        }

        /// <summary>
        /// Gets the message when <paramref name="path"/> is not found in the system.
        /// </summary>
        /// <param name="path">File system path.</param>
        /// <returns>Formated error message.</returns>
        public static string PathNotExists(string path)
        {
            return string.Format(CultureInfo.CurrentUICulture, "{0} does not exists", path);
        }
    }
}
