//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.ManagementStudioControls.RegistryManagerModule
{
    using System.Windows.Controls;
    using System.Windows.Input;
    using Microsoft.Research.ScientificWorkflow.ManagementStudioModels;
    using System.Windows;

    /// <summary>
    /// Interaction logic for TypeInitializerLeftPane.xaml
    /// </summary>
    public partial class TypeInitializerLeftPane : UserControl
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="TypeInitializerLeftPane"/> class.
        /// </summary>
        public TypeInitializerLeftPane()
        {
            InitializeComponent();
        }

        /// <summary>
        /// The selected item changed event on the treeview.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event arguments.</param>
        private void OnSelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            (this.DataContext as TypeInitializerLeftPanePresenter).TypeInitializerSelectedCommand.Execute(e.NewValue as TypeInitializerModel);
        }

        /// <summary>
        /// Event handler for the PreviewMouseDown event on the category treeview. This event is needed
        /// because the selection changed event does not fire when alternating between 2 controls.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event arguments.</param>
        private void OnGridPreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            TypeInitializerLeftPanePresenter presenter = this.DataContext as TypeInitializerLeftPanePresenter;
            if (!presenter.IsSelected && treeview.SelectedItem != null)
            {
                Grid selectedGrid = sender as Grid;
                if (selectedGrid != null)
                {
                    TypeInitializerModel model = selectedGrid.DataContext as TypeInitializerModel;

                    // If the mousedown happens on the currently selected item then fire the selected command.
                    if (treeview.SelectedItem.Equals(model))
                    {
                        presenter.TypeInitializerSelectedCommand.Execute(model);
                    }
                }
            }
        }
    }
}
