//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.ManagementStudioControls.RegistryManagerModule
{

    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using Microsoft.Research.ScientificWorkflow.ManagementStudioModels;

    /// <summary>
    /// Interaction logic for UserTree.xaml
    /// </summary>
    public partial class UserTree : UserControl
    {
        public UserTree()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Gets or sets the model.
        /// </summary>
        /// <value>The model.</value>
        public UserLeftPanePresenter Model
        {
            get { return this.DataContext as UserLeftPanePresenter; }
            set { this.DataContext = value; }
        }

        /// <summary>
        /// The selected item changed event on the treeview.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event arguments.</param>
        private void OnSelectedItemChanged(object sender, RoutedPropertyChangedEventArgs<object> e)
        {
            this.Model.ItemSelectedCommand.Execute(e.NewValue as BaseRegMgrModel);
        }

        /// <summary>
        /// Event handler for the PreviewMouseDown event on the category treeview. This event is needed
        /// because the selection changed event does not fire when alternating between 2 controls.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The event arguments.</param>
        private void Grid_PreviewMouseDown(object sender, MouseButtonEventArgs e)
        {
            if (!this.Model.IsSelected && this.userTreeview.SelectedItem != null)
            {
                Grid selectedGrid = sender as Grid;
                BaseRegMgrModel model = selectedGrid.DataContext as BaseRegMgrModel;

                // If the mousedown happens on the currently selected item then fire the selected command.
                if (this.userTreeview.SelectedItem == model)
                {
                    this.Model.ItemSelectedCommand.Execute(model);
                }
            }
        }
    }
}
