//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using System.Windows;
using System.Windows.Controls;
using Microsoft.Research.ScientificWorkflow.ManagementStudioControls.Common;
using Microsoft.Research.ScientificWorkflow.ManagementStudioModels;
using System.Windows.Input;

namespace Microsoft.Research.ScientificWorkflow.ManagementStudioControls.WorkflowManagerModule
{
    /// <summary>
    /// Interaction logic for ScheduledJobsListView.xaml
    /// </summary>
    public sealed partial class ScheduledJobsListView : UserControl
    {
        public ScheduledJobsListView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// The event handler for the context menu opening event on the listview item.
        /// </summary>
        /// <param name="sender">The source.</param>
        /// <param name="e">The event arguments.</param>
        private void ContextMenu_ContextMenuOpening(object sender, ContextMenuEventArgs e)
        {
            IListPresenter listPresenter = this.DataContext as IListPresenter;
            ListViewItem selectedItem = sender as ListViewItem;

            ContextMenu contextMenu = selectedItem.ContextMenu;
            object workflowModel = selectedItem.DataContext;

            foreach (MenuItem menuItem in listPresenter.MenuItemCollection)
            {
                ContextMenu parent = menuItem.Parent as ContextMenu;
                if (parent != null)
                {
                    parent.Items.Remove(menuItem);
                }
                menuItem.CommandParameter = workflowModel;
                contextMenu.Items.Add(menuItem);
            }
        }

        /// <summary>
        /// Event handler for the deouble click event.
        /// </summary>
        /// <param name="sender">The source.</param>
        /// <param name="e">The event arguments.</param>
        private void ListViewItemDoubleClicked(object sender, RoutedEventArgs e)
        {
            ListViewItem item = sender as ListViewItem;
            this.DisplayListView.SelectedItem = item.DataContext;

            // Invoke the trident composer on double click.
            (this.DataContext as ScheduledJobsListPresenter).EditScheduleCommand.Execute(item.DataContext as ScheduleViewModel);
        }

        /// <summary>
        /// Event handler for the delete key press event.
        /// </summary>
        /// <param name="sender">The source.</param>
        /// <param name="e">The event arguments.</param>
        private void DisplayListView_KeyDown(object sender, KeyEventArgs e)
        {
            ScheduleViewModel model = this.DisplayListView.SelectedItem as ScheduleViewModel;

            if (e.Key == Key.Delete)
            {
                if (model != null)
                {
                    (this.DataContext as ScheduledJobsListPresenter).DeleteScheduleCommand.Execute(model);
                }
            }
            // If key events to handle increase introduce a switch statement.
        }
    }
}
