//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ProvenanceSubscriberCommon.Interfaces
{
    using System;

    /// <summary>
    /// Specificiation for provenance service.
    /// </summary>
    public interface IProvenanceService
    {        
        /// <summary>
        /// Raised whenever message from blackboard is recieved.
        /// </summary>
        event EventHandler<ProvenanceModelEventArgs> ProvenanceMessage;

        /// <summary>
        /// Raised whenever  InputOutPut message from blackboard is recieved.
        /// </summary>
        event EventHandler<ProvenanceModelEventArgs> InputOutputMessage;

        /// <summary>
        /// Raised whenever  InputOutPut message from blackboard is recieved.
        /// </summary>
        event EventHandler<ProvenanceModelEventArgs> PerformanceMessage;

        /// <summary>
        /// Raised whenever  InputOutPut message from blackboard is recieved.
        /// </summary>
        event EventHandler<ProvenanceModelEventArgs> ProcessingStatusMessage;

        /// <summary>
        /// Raised whenever DataProduct message from blackboard is recieved.
        /// </summary>
        event EventHandler<ProvenanceModelEventArgs> DataProductMessage;

        /// <summary>
        /// Start the subscriber if not already started.
        /// </summary>
        void Start();

        /// <summary>
        /// Stop the subscriber if already started.
        /// </summary>
        void StopSubscriber();
    }
}
