//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Blackboard.Interfaces
{
    using System;
    using Microsoft.Research.eResearch.Blackboard.Common;

    /// <summary>
    /// Arguments for the NotificationSubscription event.
    /// </summary>
    public class NotificationSubscriptionEventArgs : EventArgs
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="NotificationSubscriptionEventArgs"/> class.
        /// </summary>
        /// <param name="returnAddress">The return address.</param>
        /// <param name="profile">The subscription profile.</param>
        public NotificationSubscriptionEventArgs(Uri returnAddress, SubscriptionProfile profile)
            : base()
        {
            this.IncomingReturnAddress = returnAddress;
            this.IncomingSubscriptionProfile = profile;
        }

        /// <summary>
        /// Gets the incoming return address.
        /// </summary>
        /// <value>The incoming return address.</value>
        public Uri IncomingReturnAddress { get; private set; }

        /// <summary>
        /// Gets the incoming subscription profile.
        /// </summary>
        /// <value>The incoming subscription profile.</value>
        public SubscriptionProfile IncomingSubscriptionProfile { get; private set; }
    }
}
