//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Blackboard.Interfaces
{
    using System;
    using Microsoft.Research.eResearch.Blackboard.Common;

    /// <summary>
    /// Arguments for the PublisherRegistering event.
    /// </summary>
    public class PublisherRegisteringEventArgs : EventArgs
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="PublisherRegisteringEventArgs"/> class.
        /// </summary>
        /// <param name="returnAddress">The return address.</param>
        public PublisherRegisteringEventArgs(Uri returnAddress)
            : base()
        {
            this.IncomingReturnAddress = returnAddress;
        }

        /// <summary>
        /// Gets the return address of the subscribing publisher.
        /// </summary>
        /// <value>The return address of the subscribing publisher.</value>
        public Uri IncomingReturnAddress { get; private set; }

        /// <summary>
        /// Gets or sets the outgoing subscription profile.
        /// </summary>
        /// <value>The outgoing subscription profile.</value>
        public SubscriptionProfile OutgoingSubscriptionProfile { get; set; }
    }
}
