//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Ontology.Lookup
{
    using System;
    using System.Globalization;

    /// <summary>
    /// Abstraction for concept value pair which will be sent to blackboard as a message.
    /// </summary>
    public class ConceptValuePair
    {
        /// <summary>
        /// Initializes a new instance of the ConceptValuePair class.
        /// </summary>
        /// <param name="concept">Name of the concept.</param>
        /// <param name="value">Concept value.</param>
        public ConceptValuePair(string concept, string value)
        {
            this.Concept = concept;
            this.Value = value;
        }

        /// <summary>
        /// Gets and sets the concept.
        /// </summary>
        public string Concept { get; private set; }

        /// <summary>
        /// Gets and sets the value.
        /// </summary>
        public string Value { get; private set; }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return String.Format(CultureInfo.CurrentUICulture, "Concept:{0}, Value:{1}", this.Concept, this.Value);
        } 
    }
}
