//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Publisher.Interfaces
{
    using System;

    /// <summary>
    /// Implementers of this interface provide code that listens for new subscription profiles
    /// from the blackboard, and fire events when new profiles are received.
    /// </summary>
    public interface IPublisherSubscriptionReceiver
    {
        /// <summary>
        /// Raised when a new subscription profile is received.
        /// </summary>
        event EventHandler<PublisherSubscriptionChangedEventArgs> PublisherSubscriptionChanged;

        /// <summary>
        /// Begins listening for new subscription profile messages.
        /// </summary>
        /// <returns>The URL of the service that is listening.</returns>
        Uri Start();

        /// <summary>
        /// Stops listening for new profile notifications.
        /// </summary>
        void Stop();
    }
}
