//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.RuntimeServices.Monitoring.Monitors
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Diagnostics;
    using System.Globalization;

    [MonitorConcept("trident:MemoryResource")]
    public class MemoryMonitor : IResourceMonitor
    {
        private PerformanceCounter memoryCounter;

        private float memoryUsage;

        public TimeSpan MonitorTimeSpan
        {
            get { return TimeSpan.FromMilliseconds(500); }
        }

        public void Setup()
        {
            // Monitor only the current process
            memoryCounter = new PerformanceCounter("Process", "Private Bytes", Process.GetCurrentProcess().ProcessName, true);
        }

        public void Teardown()
        {
            memoryCounter.Close();
            memoryCounter.Dispose();
            memoryCounter = null;
        }

        public bool MonitorResource(ActivityMonitorEvent monitorEvent)
        {
            // Sometimes the timer can schedule a call to this method on the thread pool
            // that gets invoked even after the timer is disposed (i.e. TearDown was already called).
            // If this happens, just return without asking for a message to be sent.
            if (memoryCounter == null)
            {
                return false;
            }

            // convert to KB
            memoryUsage = memoryCounter.NextValue() / 1048576;
            return true;
        }

        public string GetResource()
        {
            return memoryUsage.ToString(CultureInfo.CurrentCulture);
        }
    }
}
