//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.eResearch.Subscriber.Wcf
{
    using System;
    using Microsoft.Research.eResearch.Blackboard.Common;
    using Microsoft.Research.eResearch.Subscriber.Interfaces;
    using Microsoft.Research.eResearch.Subscriber.Wcf.WcfNotificationSubscriptionService;

    /// <summary>
    /// Provides a WCF implementation of the <see cref="INotificationSubscriptionSender"/> interface.
    /// This implementation allows a subscriber to notify the Blackboard of changes to its subscription profile.
    /// </summary>
    public class WcfNotificationSubscriptionClient : INotificationSubscriptionSender
    {
        /// <summary>
        /// Subscribes the specified return address of the subscriber using the specified
        /// subscription profile to the Blackboard.  If the subscriber has previous subscribed
        /// to the blackboard, calling Subscribe again will update (change) the subscriber's
        /// subscription profile (i.e. Subscribe can be called any number of times before
        /// calling Unsubscribe).
        /// </summary>
        /// <param name="returnAddress">The return address of the subscriber.</param>
        /// <param name="profile">The subscription profile.</param>
        public void Subscribe(Uri returnAddress, SubscriptionProfile profile)
        {
            WcfNotificationSubscriptionServiceClient client = 
                new WcfNotificationSubscriptionServiceClient("WcfNotificationSubscriptionClient");
            client.Open();            
            client.Subscribe(returnAddress, profile.ToArray());
            client.Close();
        }

        /// <summary>
        /// Unsubscribes the specified return address of the subscriber from the Blackboard.
        /// </summary>
        /// <param name="returnAddress">The return address of the subscriber.</param>
        public void Unsubscribe(Uri returnAddress)
        {
            WcfNotificationSubscriptionServiceClient client =
                new WcfNotificationSubscriptionServiceClient("WcfNotificationSubscriptionClient");
            client.Open();
            client.Unsubscribe(returnAddress);
            client.Close();
        }
    }
}
