//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow
{
    using System.Globalization;
    using System.Windows;
    using Microsoft.Win32;
    using System;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;

    /// <summary>
    /// Interaction logic for AboutBox.xaml
    /// </summary>
    public sealed partial class AboutBox : Window
    {
        /// <summary>
        /// Constructor
        /// </summary>
        public AboutBox()
        {
            InitializeComponent();
            this.GetBuildNumberFromRegistry();
        }

        /// <summary>
        /// Retrieves the current build number from windows registry.
        /// </summary>
        /// <returns>Build Number</returns>
        private void GetBuildNumberFromRegistry()
        {
            try
            {
                string buildNumber = Microsoft.Research.ScientificWorkflow.TridentUtilities.Properties.ScientificWorkFlow.BuildVersionLabel;
                RegistryKey regKey = Registry.LocalMachine.OpenSubKey(Constants.PRODUCT_VERSION_REGISTRY_PATH);
                if (null != regKey)
                {
                    this.VersionText.Text = string.Format(CultureInfo.CurrentCulture, "{0}: {1}", buildNumber, regKey.GetValue(Constants.PRODUCT_VERSION_REGISTRY_KEY).ToString());
                }
            }
            catch (Exception exception)
            {
                TridentErrorHandler.HandleUIException(exception);
            }
        }
    }
}
