//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using SR = Microsoft.Research.DataLayer;
using System;

namespace Microsoft.Research.ScientificWorkflow.TridentModel
{
    /// <summary>
    /// A model which contains the additional parameter information.
    /// </summary>
    public class AdditionalParametersModel
    {
        /// <summary>
        /// The domain.
        /// </summary>
        private string domain;

        /// <summary>
        /// The parameter name.
        /// </summary>
        private string parameter;

        /// <summary>
        /// The parameter name.
        /// </summary>
        private string parameterValue;

        /// <summary>
        /// Initializes a new instance of additional parameter model.
        /// </summary>
        public AdditionalParametersModel()
        {
        }

        /// <summary>
        /// Initializes a new instance of additional parameter model.
        /// </summary>
        /// <param name="additionalParameters">The registry object from where the information is to be retrieved.</param>
        public AdditionalParametersModel(SR.AdditionalParameter additionalParameters)
        {
            if (additionalParameters == null)
            {
                throw new ArgumentNullException("additionalParameters");
            }

            this.domain = additionalParameters.Domain;
            this.parameter = additionalParameters.ParamName;
            this.parameterValue = additionalParameters.ParamValue;
        }

        /// <summary>
        /// Gets or Sets the domain.
        /// </summary>
        public string Domain
        {
            get
            {
                return this.domain;
            }
            set
            {
                this.domain = value;
            }
        }

        /// <summary>
        /// Gets or Sets the Parameter name.
        /// </summary>
        public string Parameter
        {
            get
            {
                return this.parameter;
            }
            set
            {
                this.parameter = value;
            }
        }

        /// <summary>
        /// Gets or Sets the parameter value.
        /// </summary>
        public string ParameterValue
        {
            get
            {
                return this.parameterValue;
            }
            set
            {
                this.parameterValue = value;
            }
        }
    }
}
