//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.TridentModel
{
    using System;
    using System.Workflow.ComponentModel;
    using ER = Microsoft.Research.DataLayer;

    [Serializable()]
    public class IfElseActivityCompositeModel : CompositeActivityModel
    {
        /// <summary>
        /// Constructor of Activity Model.
        /// </summary>
        /// <param name="erActivity">Activity for which the model has to be created.</param>
        public IfElseActivityCompositeModel(ER.ActivitySequence erActivitySequence, ModelFactory modelFactory)
            : base(erActivitySequence, modelFactory) { }

        /// <summary>
        /// Constructor of Activity Model.
        /// </summary>
        /// <param name="source">Activity metadata for the activity model.</param>
        public IfElseActivityCompositeModel(ActivityMetadata source, string uniqueCount)
            : base(source, uniqueCount) { }

        /// <summary>
        /// Create the activity model for the Activity object.
        /// </summary>
        /// <param name="activity">
        /// The ComponentModel.Activity object.
        /// </param>
        public IfElseActivityCompositeModel(Activity activity, ModelFactory modelFactory)
            : base(activity, modelFactory) { }

        /// <summary>
        /// Create the activity model for the Activity object.
        /// </summary>
        /// <param name="activity">
        /// The ComponentModel.Activity object.
        /// </param>
        public IfElseActivityCompositeModel(Activity activity, BaseCompositeModel parentModel, ModelFactory modelFactory)
            : base(activity, parentModel, modelFactory) { }

        /// <summary>
        ///  Constructor of Activity Model.
        /// </summary>
        /// <param name="erActivitySequence">Activity for which the model has to be created.</param>
        /// <param name="parentModel">Parent model of teh Activity Sequence.</param>
        public IfElseActivityCompositeModel(ER.ActivitySequence erActivitySequence, BaseCompositeModel parentModel, ModelFactory modelFactory)
            : base(erActivitySequence, parentModel, modelFactory) { }

        /// <summary>
        /// Initializes a new instance of the <see cref="IfElseActivityCompositeModel"/> class.
        /// </summary>
        /// <param name="updatedActivity">The updated activity.</param>
        /// <param name="parentModel">The parent model.</param>
        /// <param name="uniqueCount">The unique count.</param>
        public IfElseActivityCompositeModel(ER.Activity updatedActivity, BaseCompositeModel parentModel, string uniqueCount)
            : base(updatedActivity, parentModel, uniqueCount) { }

        /// <summary>
        /// Gets true if the concrete implementation is an ifElse model.
        /// </summary>
        public override bool IsIfElseActivityModel
        {
            get
            {
                return true;
            }
        }

        public override void Accept(TridentBaseVisitor visitor)
        {
            visitor.VisitIfElseActivityModel(this);
        }
    }
}
