//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using Microsoft.Research.ScientificWorkflow.TridentModel;
using Microsoft.Research.ScientificWorkflow.UIDesigner.ActivityImportWizards;

    /// <summary>
    /// Event args encapsulating the model.
    /// </summary>
    public class ImportEventArgs : EventArgs
    {
        /// <summary>
        /// Gets or sets the exception message.
        /// </summary>
        public string ErrorMessage { get; set; }

        /// <summary>
        /// Gets or sets the import status.
        /// </summary>
        public bool Status { get; set; }

        /// <summary>
        /// Initializes a new instance of the <see cref="ModelEventArgs"/> class.
        /// </summary>
        /// <param name="model">The model.</param>
        public ImportEventArgs(string msg, bool result)
        {
            this.ErrorMessage = msg;
            this.Status = result;
        }
    }

    /// <summary>
    /// Event args encapsulating the next screen.
    /// </summary>
    public class ActivityImportEventArgs : EventArgs
    {
        /// <summary>
        /// Gets or sets the next screen.
        /// </summary>
        public IActivityWizardScreens NextScreen
        {
            get;
            set;
        }

        /// <summary>
        /// Activity import args.
        /// </summary>
        /// <param name="nextScreen">
        /// NextScreen.
        /// </param>
        public ActivityImportEventArgs(IActivityWizardScreens nextScreen)
        {
            this.NextScreen = nextScreen;
        }
    }
}