//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.UIDesigner.Monitor
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Data;
    using System.Windows.Documents;
    using System.Windows.Input;
    using System.Windows.Media;
    using System.Windows.Media.Imaging;
    using System.Windows.Navigation;
    using System.Windows.Shapes;
    using Microsoft.Research.ScientificWorkflow.TridentUtilities;
    using Microsoft.Research.DataLayer;
    using System.Diagnostics;
    using System.IO;

    /// <summary>
    /// Interaction logic for DataProductsView.xaml
    /// </summary>
    public sealed partial class DataProductsView : UserControl
    {
        /// <summary>
        /// Constructor.
        /// </summary>
        public DataProductsView()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Get and sets DataProductPresentationModel
        /// </summary>
        public DataProductPresentationModel Model
        {
            get { return this.DataContext as DataProductPresentationModel; }
            set { this.DataContext = value; }
        }

        /// <summary>
        /// Fired on link click.
        /// </summary>
        /// <param name="sender">
        /// Link.
        /// </param>
        /// <param name="e">
        /// MouseButtonEventArgs.
        /// </param>
        private void OnLinkClick(object sender, MouseButtonEventArgs e)
        {
            FrameworkElement link = sender as FrameworkElement;
            if (link != null)
            {
                DataProductItem dataproductView = ((ContentPresenter)(link.TemplatedParent)).DataContext as DataProductItem;
                if (dataproductView != null && dataproductView.OriginalDataProduct != null)
                {
                    dataproductView = DataProductPresentationModel.GetDataProductItem(dataproductView.OriginalDataProduct);
                    TextBlock linkTxtBlk = link as TextBlock;
                    if (linkTxtBlk != null)
                    {
                        linkTxtBlk.Foreground = Brushes.BlueViolet;
                    }

                    try
                    {
                        Process.Start(dataproductView.HtmlPath);
                    }
                    catch (IOException ex)
                    {
                        TridentErrorHandler.HandleUIException(ex);
                    }
                    catch (System.ComponentModel.Win32Exception ex)
                    {
                        TridentErrorHandler.HandleUIException(ex);
                    }
                    catch (ObjectDisposedException ex)
                    {
                        TridentErrorHandler.HandleUIException(ex);
                    }
                    catch (Exception ex)
                    {
                        TridentErrorHandler.HandleUIException(ex);
                    }
                }
            }
        }
    }
}
