//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.TridentModelSL
{
    using System;
    using System.Collections.ObjectModel;

    /// <summary>
    /// The category model.
    /// </summary>
    public class LeftPaneModel
    {
        #region Private Variables

        /// <summary>
        /// Name of the model
        /// </summary>
        private string name;

        /// <summary>
        /// Label of the model
        /// </summary>
        private string label;

        /// <summary>
        /// Id of the model
        /// </summary>
        private Guid id;

        #endregion Private Variables

        /// <summary>
        /// The child models.
        /// </summary>
        private Collection<LeftPaneModel> children = new Collection<LeftPaneModel>();

        /// <summary>
        /// Gets the children.
        /// </summary>
        /// <value>The children.</value>
        public Collection<LeftPaneModel> Children
        {
            get
            {
                return this.children;
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether this instance is category model.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is category model; otherwise, <c>false</c>.
        /// </value>
        public bool IsCategoryModel { get; private set; }

        /// <summary>
        /// Gets or sets the version label.
        /// </summary>
        /// <value>The version label.</value>
        public string VersionLabel { get; private set; }

        /// <summary>
        /// The modified time of the workflow.
        /// </summary>
        public DateTime ModifiedTime { get; private set; }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name
        {
            get
            {
                return this.name;
            }

            set
            {
                this.name = value;
            }
        }

        /// <summary>
        /// Gets or sets the label.
        /// </summary>
        /// <value>The label.</value>
        public string Label
        {
            get
            {
                return this.label;
            }

            set
            {
                this.label = value;
            }
        }

        /// <summary>
        /// Gets or sets the id.
        /// </summary>
        /// <value>The id.</value>
        public Guid Id
        {
            get
            {
                return this.id;
            }

            set
            {
                this.id = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="LeftPaneModel"/> class.
        /// </summary>
        public LeftPaneModel()
        {
        }
    }
}
