//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

namespace Microsoft.Research.ScientificWorkflow.TridentModelSL
{
    using System;
    using System.Globalization;
    using service = Microsoft.Research.ScientificWorkflow.TridentSilverlightService;

    /// <summary>
    /// Respresents a workflow for a UI element
    /// </summary>
    public sealed class WorkflowModel : BaseModel
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowModel"/> class.
        /// </summary>
        /// <param name="source">The source.</param>
        public WorkflowModel(service.LeftPaneModel source)
        {
            this.VersionLabel = source.versionLabel;
            this.Name = source.name;
            this.Id = source.id;
            this.Label = source.label;

            this.ModifiedTime = source.modifiedTime;

            this.ItemIcon = BaseModel.CreateImage(@"../../Images/WorkflowIcon.png");
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowModel"/> class.
        /// </summary>
        /// <param name="source">The source.</param>
        public WorkflowModel(service.WorkflowModel source)
        {
            this.VersionLabel = source.versionLabel;
            this.Name = source.name;
            this.Id = source.id;
            this.Label = source.label;

            this.ModifiedTime = source.modifiedTime;

            this.ItemIcon = BaseModel.CreateImage(@"../../Images/WorkflowIcon.png");
        }

        /// <summary>
        /// Gets or sets the version label.
        /// </summary>
        /// <value>The version label.</value>
        public string VersionLabel { get; private set; }

        /// <summary>
        /// The modified time of the workflow.
        /// </summary>
        public DateTime ModifiedTime { get; private set; }

        /// <summary>
        /// Gets or sets the label.
        /// </summary>
        /// <value>The label.</value>
        public override string Label
        {
            get
            {
                return string.IsNullOrEmpty(this.VersionLabel) ? this.Name : string.Format(CultureInfo.CurrentUICulture, "{0} ({1})", this.Name, this.VersionLabel);
            }
            set
            {
                base.Label = value;
            }
        }

        /// <summary>
        /// Sorts the activity list based on workflow name and the modified time.
        /// Implementation of Icomparer object
        /// </summary>
        /// <param name="activityA">Activity</param>
        /// <param name="activityB">Activity</param>
        /// <returns></returns>
        public static int WorkflowComparer(WorkflowModel first, WorkflowModel second)
        {
            int result = 0;
            string firstStringToCompare = first.Name;
            string secondStringToCompare = second.Name;
            int compareValue = string.Compare(firstStringToCompare, secondStringToCompare, StringComparison.OrdinalIgnoreCase);
            if (compareValue != 0)
            {
                result = compareValue > 0 ? 1 : -1;
            }
            else
            {
                compareValue = DateTime.Compare(first.ModifiedTime, second.ModifiedTime);
                result = compareValue > 0 ? -1 : 1;
            }
            return result;
        }

    }
}
