//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//*********************************************************

using System.Collections.ObjectModel;

namespace Microsoft.Research.ScientificWorkflow.TridentModelSL
{
    /// <summary>
    /// Respresents a workflow Output for UI elements
    /// </summary>
    public class WorkflowOutputActivityModel
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowOutputActivityModel"/> class.
        /// </summary>
        /// <param name="activityName">Name of the activity.</param>
        /// <param name="activityUniqueId">The activity unique id.</param>
        public WorkflowOutputActivityModel(string activityName, string activityUniqueId)
        {
            this.ActivityName = activityName;
            this.ActivityUniqueName = activityUniqueId;
            this.DataProducts = new ObservableCollection<WorkflowOutputModel>();
        }

        /// <summary>
        /// Gets or sets the name of the activity.
        /// </summary>
        /// <value>The name of the activity.</value>
        public string ActivityName { get; private set; }

        /// <summary>
        /// Gets or sets the dataproducts.
        /// </summary>
        /// <value>The dataproducts.</value>
        public ObservableCollection<WorkflowOutputModel> DataProducts{get ; private set;}

        /// <summary>
        /// Gets or sets the name of the activity unique.
        /// </summary>
        /// <value>The name of the activity unique.</value>
        public string ActivityUniqueName { get; private set; }
    }
}
