//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//********************************************************* 

using System.Collections.Generic;

namespace Microsoft.TridentWordAddIn.Common
{
    public enum ContentItemType
    {
        Activity,
        Workflow,
        ResultFile
    }

    public class ContentItem
    {
        private List<string> belongsTo;

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentItem"/> class.
        /// </summary>
        public ContentItem()
        {
            this.belongsTo = new List<string>();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ContentItem"/> class.
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="size">The size.</param>
        /// <param name="type">The type.</param>
        public ContentItem(string name, long size, ContentItemType type)
        {
            this.belongsTo = new List<string>();
            this.Name = name;
            this.Size = size;
            this.ContentType = type;
        }

        /// <summary>
        /// Gets or sets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name { get; set; }

        /// <summary>
        /// Gets or sets the size.
        /// </summary>
        /// <value>The size.</value>
        public long Size { get; set; }

        /// <summary>
        /// Gets or sets the type.
        /// </summary>
        /// <value>The type.</value>
        public ContentItemType ContentType { get; set; }

        /// <summary>
        /// Gets the belongs to.
        /// </summary>
        /// <value>The belongs to.</value>
        public List<string> BelongsTo
        {
            get { return this.belongsTo; }
        }
    }
}
