//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//********************************************************* 

using System.Collections.Generic;

namespace Microsoft.TridentWordAddIn.Common
{
    public class RerunWorkflowInformation
    {
        private string jobName;
        private string machineName;
        private Workflow rerunWorkflow;
        private List<WorkflowParameter> workflowParameters;

        /// <summary>
        /// Initializes a new instance of the <see cref="RerunWorkflowInformation"/> class.
        /// </summary>
        /// <param name="jobName">Name of the job.</param>
        /// <param name="machineName">Name of the machine.</param>
        /// <param name="workflow">The workflow.</param>
        /// <param name="jobParameters">The job parameters.</param>
        public RerunWorkflowInformation(string jobName, string machineName, Workflow workflow, List<WorkflowParameter> workflowParameters)
        {
            this.rerunWorkflow = workflow;
            this.jobName = jobName;
            this.machineName = machineName;
            this.workflowParameters = workflowParameters;
        }

        /// <summary>
        /// Gets the name of the job.
        /// </summary>
        /// <value>The name of the job.</value>
        public string JobName
        {
            get { return this.jobName; }
        }

        /// <summary>
        /// Gets the name of the machine.
        /// </summary>
        /// <value>The name of the machine.</value>
        public string MachineName
        {
            get { return this.machineName; }
        }

        /// <summary>
        /// Gets the rerun workflow.
        /// </summary>
        /// <value>The rerun workflow.</value>
        public Workflow Workflow
        {
            get { return this.rerunWorkflow; }
        }

        /// <summary>
        /// Gets the job parameters.
        /// </summary>
        /// <value>The job parameters.</value>
        public List<WorkflowParameter> WorkflowParameters
        {
            get { return this.workflowParameters; }
        }
    }
}
