//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//********************************************************* 

using Microsoft.TridentWordAddIn.Common;

namespace Microsoft.TridentWordAddIn.Presentation
{
    internal class AssociateWorkflowCommand : RibbonCommand
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="AssociateWorkflowCommand"/> class.
        /// </summary>
        /// <param name="controller">The controller.</param>
        public AssociateWorkflowCommand(IController controller)
            : base(controller)
        {
        }

        /// <summary>
        /// Invokes the command.
        /// </summary>
        /// <param name="controlId">The originating control id.</param>
        public override void Invoke(string controlId)
        {
            this.Invoke(true);
        }

        /// <summary>
        /// Invokes the specified is pressed.
        /// </summary>
        /// <param name="isPressed">if set to <c>true</c> [is pressed].</param>
        public override void Invoke(bool isPressed)
        {
            if (!ControllerInstance.CurrentConnectionName.IsNullOrEmpty())
            {
                TridentConnection connection = this.ControllerInstance.UserData.Connections[ControllerInstance.CurrentConnectionName];
                TridentConnection connectionReturn = null;
                if (!this.ControllerInstance.TestConnectionAndPromptIfRequired(connection, ref connectionReturn, true))
                {
                    this.ControllerInstance.RemoveConnection(connection.Name);
                    this.ControllerInstance.ConnectionChanged(false, ConnectionOperation.Deleted);
                    this.ControllerInstance.SetNextDefaultConnection();
                }
            }

            base.ShowTaskPane(TaskPaneKey.AssociateWorkflow, isPressed);
        }
    }
}
