//*********************************************************
//
//    Copyright (c) Microsoft. All rights reserved.
//    This code is licensed under the Apache License, Version 2.0.
//    THIS CODE IS PROVIDED *AS IS* WITHOUT WARRANTY OF
//    ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING ANY
//    IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
//    PURPOSE, MERCHANTABILITY, OR NON-INFRINGEMENT.
//
//********************************************************* 

using System;
using System.Windows.Forms;
using Microsoft.TridentWordAddIn.Common;

namespace Microsoft.TridentWordAddIn.Presentation
{
    internal partial class CloudDatabaseAccountInformation : UserControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="CloudDatabaseAccountInformation"/> class.
        /// </summary>
        public CloudDatabaseAccountInformation()
        {
            this.InitializeComponent();
            this.userNamePassword.OnValidation += new EventHandler<ConnectionValidateEventArgs>(userNamePassword_OnValidation);
        }

        #endregion

        /// <summary>
        /// Occurs when [on validation].
        /// </summary>
        public event EventHandler<ConnectionValidateEventArgs> OnValidation;

        #region Properties

        /// <summary>
        /// Gets or sets the name of the user.
        /// </summary>
        /// <value>The name of the user.</value>
        public string UserName
        {
            get
            {
                return this.userNamePassword.UserName;
            }
            set
            {
                this.userNamePassword.UserName = value;
            }
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>The password.</value>
        public string Password
        {
            get
            {
                return this.userNamePassword.Password;
            }
            set
            {
                this.userNamePassword.Password = value;
            }
        }

        #endregion

        /// <summary>
        /// Handles the OnValidation event of the userNamePassword control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.TridentWordAddIn.Common.ConnectionValidateEventArgs"/> instance containing the event data.</param>
        public void userNamePassword_OnValidation(object sender, ConnectionValidateEventArgs e)
        {
            if (this.OnValidation != null)
            {
                this.OnValidation(sender, e);
            }
        }

        /// <summary>
        /// Determines whether this instance is valid.
        /// </summary>
        /// <returns></returns>
        public ValidationResult IsValid()
        {
            return this.userNamePassword.IsValid();
        }
    }
}
